'use strict';

angular.module('adf.widget.electre.lavoisier')
.directive('tableDisplay',
    [
        "lavoisierHttpRequestService", "parserService", "NgTableParams", "LAVOISIER_TEMPLATES_PATH", "$sce",
        function (lavoisierHttpRequestService, parserService, NgTableParams, LAVOISIER_TEMPLATES_PATH, $sce) {
            /**
             * Correspond à la propriété dans laquelle est stocké la valeur au sein des objets.
             * @type String
             */
            var jsonPropertyToTakeFromRequest = "__text";
            /**
             * Dit si toutes les tables ont exactement les même collonnes.
             * @type Boolean
             */
            var allTableSameColumn = true;
            /**
             * Contient les noms des collonnes.
             * @type Array<String>
             */
            var columns_name = [];
            /**
             * Contient les collonnes spécifier dans les options (options.x.axis.column) qui doivent être les premières collonnes que l'on affiche.
             * @type Array<String>
             */
            var firstColumns;
            /**
             * Tableau de définission des collonnes pour ngTable.
             * @type Array<Object>
             */
            var columns;
            /**
             * Tableau de donnée pour ngTable.
             * @type Array<Object>
             */
            var rows;
            /**
             * Contient le nom du fichier de template.
             * @type String
             */
            var templateUrl;
            /**
             * Dit si les filtres sont autorisé.
             * @type Boolean
             */
            var isFilterOn;
            /**
             * Dit si la valeur est accompagné d'un lien (false par défaut).
             * @type Boolean
             */
            var isLinkValue = false;

            /**
             * Donne le titre de la collonne.
             * @param {String} column
             * @returns {String}
             */
            function getTitle (column) { return column.substring(0, 1).toUpperCase() + column.substring(1).toLowerCase(); }
            /**
             * Analyse le JSON pour détecter si oui ou non la valeur à afficher est accompagné d'un lien.
             * Return {Boolean}
             */
            function isLinkedValue(table) {
                return table.row[0].column[0]["_link"] !== undefined;
            }
            /**
             * Donne la valeur à afficher (sans lien).
             * @param {Object} column La collonne que l'on souhaite affiché.
             * @param {Object} row La ligne que l'on souhaite affiché.
             */
            function normalValue (column, row) {
                var html = '<p>' + row[column.field].value + '</p>';
                return $sce.trustAsHtml(html);
            }
            /**
             * Donne la valeur à afficher (avec lien).
             * @param {Object} column La collonne que l'on souhaite affiché.
             * @param {Object} row La ligne que l'on souhaite affiché.
             */
            function linkValue(column, row) {
                var object = row[column.field];
                var html = "<p><a href='" + object.link + "' target='_blank'>" + object.value + "</a></p>";

                return $sce.trustAsHtml(html);
            }

            /**
             * Crée le tableau des premières collonnes que l'on doit affiché à partir des options données par la requête.
             * @param {Object} options
             */
            function createColumnsFromOptions (options) {
                /*On vérifie qu'il y a des options dans options.x.axis.
                Ici on ne teste que options car s'il y a des options, la feuille de définission .xsd dit clairement que :
                             - dans <options> il y a UN <x>;
                             - dans <x> il y a AU MOINS UN <axis>.*/
                if (options) {
                    firstColumns = [];
                    if (options.x.axis.length) createColumnsFromMultipleAxis(options.x.axis);
                    else createColumnsFromSingleAxis(options.x.axis);
                }
            }
            /**
             * Crée le tableau des premières collonnes que l'on doit affiché à partir de plusieurs xAxis.
             * @param {Array<Object>} xAxis_arr
             */
            function createColumnsFromMultipleAxis (xAxis_arr) {
                for (var a = 0 ; a < xAxis_arr.length ; a++) createColumnsFromSingleAxis(xAxis_arr[a]);
            }
            /**
             * Crée le tableau des premières collonnes que l'on doit affiché à partir d'un seul xAxis.
             * @param {Object} xAxis
             */
            function createColumnsFromSingleAxis (xAxis) {
                if (xAxis.column.length) addMultipleAxisColumn(xAxis.column);
                else addSingleAxisColumn(xAxis.column);
            }
            /**
             * Ajoute plusieurs collonne à 'firstColumns'.
             * @param {Array<Object>} column_arr
             */
            function addMultipleAxisColumn (column_arr) {
                for (var c = 0 ; c < column_arr.length ; c++) {
                    addSingleAxisColumn(column_arr[c]);
                }
            }
            /**
             * Ajoute une collonne à 'firstColumns'.
             * @param {Object} column
             */
            function addSingleAxisColumn (column) {
                firstColumns.push(column[jsonPropertyToTakeFromRequest]);
            }

            /**
             * Crée le tableau des collonnes depuis la/les table donnée par la requête.
             * @param {Array<Object>/Object} table
             */
            function createColumnsFromRequestTable (table) {
                columns = []; columns_name = [];

                if (table.length)   createColumnsFromTables(table);
                else                createColumnsFromTable(table);
            }
            /**
             * Crée le tableau de collonne depuis plusieurs table.
             * @param {Array<Object>} table_arr
             */
            function createColumnsFromTables (table_arr) {
                columns.push({
                    field     : "table_group_title",
                    title     : "Table",
                    sortable  : "table_group_title",
                    show      : false,
                    groupable : "table_group_title"
                });

                for (var t = 0 ; t < table_arr.length ; t++)
                    if (allTableSameColumn) createColumnsFromTable(table_arr[t], t);
                    else break;
            }
            /**
             * Crée le tableau de collonne depuis une table.
             * @param {Object} table
             * @param {Number} [table_id] Utilisé s'il y a plusieurs table. Identifiant de la table. Utilisé à partir de la seconde table pour vérifier que toutes les tables possède exactement les mêmes collonnes.
             */
            function createColumnsFromTable (table, table_id) {

                if (table.column_labels.column_label.length)    addColumnsFromMultipleTableColumn (table.column_labels.column_label, table_id);
                else                                            addColumnsFromSingleTableColumn (table.column_labels.column_label, table_id);
            }
            /**
             * Ajoute plusieurs collonne à 'columns'.
             * @param {Array<Object>} table_columns
             * @param {Number} [table_id] Utilisé s'il y a plusieurs table. Identifiant de la table. Utilisé à partir de la seconde table pour vérifier que toutes les tables possède exactement les mêmes collonnes.
             */
            function addColumnsFromMultipleTableColumn (table_columns, table_id) {
                for (var c = 0 ; c < table_columns.length ; c++)
                    if (allTableSameColumn) addColumnsFromSingleTableColumn(table_columns[c], table_id);
                    else break;
            }
            /**
             * Ajoute une collonne à 'columns' qui servira comme définition de collonne à ngTable.
             * @param {Object} table_column
             * @param {Number} [table_id] Utilisé s'il y a plusieurs table. Identifiant de la table. Utilisé à partir de la seconde table pour vérifier que toutes les tables possède exactement les mêmes collonnes.
             */
            function addColumnsFromSingleTableColumn (table_column, table_id) {
                /*S'il y a plusieurs table.*/
                if (table_id && table_id !== 0) {
                    /*Vérifie qu'elles possèdent toutes les mêmes collonnes.*/
                    if (columns_name.indexOf(table_column[jsonPropertyToTakeFromRequest]) === -1) {
                        allTableSameColumn = false;
                    }
                } else {
                    var filter = {}; filter[[table_column[jsonPropertyToTakeFromRequest]] + ".value"] = "text";
                    columns.push({
                        field : table_column[jsonPropertyToTakeFromRequest],
                        title : getTitle(table_column[jsonPropertyToTakeFromRequest]),
                        sortable : table_column[jsonPropertyToTakeFromRequest],
                        show : true
                    });
                    /*Ajout des filtres s'ils sont demandé.*/
                    if (isFilterOn) columns[columns.length - 1].filter = filter;
                    /*Formattage de la cellule (avec/sans lien).*/
                    if (isLinkValue)    columns[columns.length -1].getValue = linkValue;
                    else                columns[columns.length -1].getValue = normalValue;

                    columns_name.push(table_column[jsonPropertyToTakeFromRequest]);
                }
            }

            /**
             * Crée le tableau de données depuis la/les table donnée par la requête.
             * @param {Array<Object>/Object} table
             */
            function createRowsFromRequestTable (table) {
                rows = [];

                if (table.length)   createRowsFromTables(table);
                else                createRowsFromTable(table);
            }
            /**
             * Crée un tableau de rows depuis plusieurs table.
             * @param {Array<Object>} tables
             */
            function createRowsFromTables (tables) {
                for (var t = 0 ; t < tables.length ; t++) createRowsFromTable(tables[t], true);
            }
            /**
             * Crée un tableau de rows depuis une table.
             * @param {Object} table
             * @param {boolean} isMultipleTable Dit s'il y a plusieurs table.
             */
            function createRowsFromTable (table, isMultipleTable) {
                if (table.row.length) addRowsFromMultipleTableRow(table.row, (isMultipleTable ? table["_title"] : undefined));
                else addRowFromSingleTableRow(table.row, (isMultipleTable ? table["_title"] : undefined));
            }
            /**
             * Ajoute les 'row' de la table dans le tableau.
             * @param {Array<Object>} table_rows
             * @param {String} [table_title] Nom de la table dans laquel ce trouve les lignes. Utilisé s'il y a plusieurs table à afficher.
             */
            function addRowsFromMultipleTableRow(table_rows, table_title) {
                for (var tr = 0 ; tr < table_rows.length ; tr++) addRowFromSingleTableRow(table_rows[tr], table_title);
            }
            /**
             * Ajoute une ligne dans le tableau.
             * @param {Object} table_row
             * @param {String} [table_title] Nom de la table dans laquel ce trouve la ligne. Utilisé s'il y a plusieurs table à afficher.
             */
            function addRowFromSingleTableRow(table_row, table_title) {
                if (table_row.column.length) addRowFromMultipleColumnFromTableRow(table_row.column, table_title);
                else  {
                    var rowToAdd = (table_title ? {table_group_title : table_title} : {});
                    addRowFromSingleColumnFromTableRow(rowToAdd, columns_name[0], table_row.column);

                    rows.push(rowToAdd);
                }
            }
            /**
             * Ajoute toutes les collonnes de la ligne au tableau.
             * @param {Array<Object>} table_row_columns
             * @param {String} [table_title] Nom de la table dans laquel ce trouve les collonnes. Utilisé s'il y a plusieurs table à afficher.
             */
            function addRowFromMultipleColumnFromTableRow(table_row_columns, table_title) {
                var rowToAdd = (table_title ? {table_group_title : table_title} : {});
                /*Création collonne par collonne de la ligne.*/
                for (var trc = 0 ; trc < table_row_columns.length ; trc++) {
                    addRowFromSingleColumnFromTableRow (rowToAdd, columns_name[trc], table_row_columns[trc]);
                }
                /*Ajout de la ligne dans la liste rows.*/
                rows.push(rowToAdd);
            }
            /**
             * Ajoute une collonne à la ligne donnée.
             * @param {Object} row
             * @param {String} column_name
             * @param {Object} column
             */
            function addRowFromSingleColumnFromTableRow (row, column_name, column) {
                row[column_name] = { value : column[jsonPropertyToTakeFromRequest] };
                if (isLinkValue) row[column_name].link = column["_link"];
            }

            /**
             * Trie les éléments de columns en se basant sur le tableau firstColumns.
             */
            function trieColumn () {
                columns.sort(function (a, b) {
                    var index_a = firstColumns.indexOf(a.field);
                    var index_b = firstColumns.indexOf(b.field);
                    if (index_a !== -1) {
                        if (index_b !== -1) {
                            return index_a - index_b;
                        } else {
                            return -1;
                        }
                    } else if (index_b !== -1) {
                        return 1;
                    } else {
                        return 0;
                    }
                });
            }

            return {
                scope : {
                    control : '='
                },
                link : function (scope, element, attrs) {
                    isFilterOn = scope.control.tableConfig.filter;
                    var options, table;
                    scope.valid = false;
                    scope.error = false;
                    scope.control.processing();

                    lavoisierHttpRequestService.getResult_2(scope.control.httpConfig)
                        .then(
                            function (value) {
                                /*Parse la value depuis un String XML vers du JSON.*/
                                var json = parserService.xml_str2json(value);
                                /*Détection du lien qui accompagne la valeur.*/
                                if (json.tables.table.length)    isLinkValue = isLinkedValue(json.tables.table[0]);
                                else                             isLinkValue = isLinkedValue(json.tables.table);

                                /*Récupération des options et de la/des table.*/
                                options = json.tables.options; table = json.tables.table;
                                /*Récupération des informations de collonne depuis : les options et la/les table.*/
                                createColumnsFromOptions(options);
                                createColumnsFromRequestTable(table);
                                /*Trie si nécessaire*/
                                if (firstColumns) trieColumn();
                                /*Récupération des données depuis la/les table.*/
                                createRowsFromRequestTable(table);

                                if (table.length && allTableSameColumn) {
                                    scope.table = {
                                        columns : columns,
                                        tableParams : new NgTableParams(
                                            {
                                                group : {
                                                    table_group_title : "asc"
                                                }
                                            },{
                                                groupOptions : {
                                                    isExpanded : false
                                                },
                                                dataset : rows
                                            }
                                        )
                                    }
                                } else {
                                    scope.table = {
                                        columns : columns,
                                        tableParams : new NgTableParams({}, {dataset : rows})
                                    };
                                }
                            }, function (error) {
                                scope.requestFailed = true;
                                throw "An error has occurred during the request. Please verify you have entered valid info.";
                            }
                        ).then(function () {
                            /*Indication sur le template html à utilisé.*/
                            if (table.length)   templateUrl = "multi-table.html";
                            else                templateUrl = "one-table.html";

                            scope.control.success();
                            scope.valid = true;
                        }, function (error) {
                            if (scope.requestFailed)        scope.control.requestFailed();
                            else                            scope.control.processingFailed();
                        });

                    scope.getContentUrl = function () {
                        if (scope.valid)        return LAVOISIER_TEMPLATES_PATH + "/table/directives/" + templateUrl;
                        else                    return false;
                    };
                },
                template : "<div ng-include='getContentUrl()'></div>"
            };
        }
    ]
);